<?php
class Pm_Rvc_Shortcode {
	public function init() {
		// Initialize shortcode backend button
		add_action( 'admin_init', array( $this, 'init_plugin' ) );
		// Shortcode frontend output
		add_shortcode( 'pm_revisitorc', array( $this, 'pm_output_shortcode' ) );
	}

	public function init_plugin() {
		// Don't bother if user does not have permissions
		if ( current_user_can('edit_posts') && current_user_can('edit_pages') ) {
			add_filter( 'mce_external_plugins', array( $this, 'tmce_plugin' ) );
			add_filter( 'mce_buttons', array( $this, 'tmce_buttons' ) );
			add_action ( 'admin_footer', array( $this, 'pm_rvc_shortcode_modal_dialog' ) );
		}
	}

	public function pm_output_shortcode( $atts = array(), $content = null ) {
		extract( shortcode_atts( array(
			'trackr_id' => ''
			), $atts ));
		$pm_rvc_opts = get_option( 'pm_rvc_options' );
		$plugin_name = PM_RVC_PLUGIN_NAME;

		// Are we viewing from admin? If so, don't output
		if( is_admin() ) {
			return <<<HTML
			<div class="pm-rvc-tinymce-holder">{$plugin_name} : {$trackr_id}</div>
HTML;
		} else {
		// Cookies logic and setting already done during init (function.php)
		if( isset( $_COOKIE['pmrvc'] ) ) {
			// Get trackr_id from cookie and display corresponding content
			$cookie_values = json_decode(stripslashes($_COOKIE['pmrvc']));

			// Check when cookie expires, if above 24 hours
			if( ( $cookie_values->expire > time() + 86400 ) && ( $cookie_values->trackr_id == 'returning' ) ) {
				// Show default content (not used removed)				
			} else {
				wp_enqueue_style( 'pm-rvc-front', PM_RVC_DIR.'css/front.css', array(), PM_RVC_VERSION, 'all' );
				$trackr_id = ( $cookie_values->trackr_id == $trackr_id ) ? esc_attr( $cookie_values->trackr_id ) : '' ;
				if( $trackr_id ) {
					$tracker_content = $this->find_content( $trackr_id, $pm_rvc_opts['tracker_content'] );
					$copy = stripslashes( $tracker_content['content'] );
					return <<<HTML
					<div class="pm_rvc_content {$trackr_id}">
					{$copy}
					</div>
HTML;
	
					}
				}
		}
		}
	}

	protected function find_content( $trackr_id, $tracker_content ) {
		foreach ($tracker_content as $content) {
			if( $content['id'] === $trackr_id ) {
				return $content;
				break;
			}
		}
	}

	public function tmce_plugin( $plugins ) {
		$plugins['pmrvc_button'] = PM_RVC_DIR.'js/tinymce.js';
		return $plugins;
	}

	public function tmce_buttons( $buttons ) {
		array_push( $buttons, 'pm_rvc_mce_button' );
    	return $buttons;
	}

	public function pm_rvc_shortcode_modal_dialog() {
		// Only load on edit screens
		$currScreen = get_current_screen();
		if( !empty( $currScreen ) && $currScreen -> base == 'post' ) {
		$pm_rvc_opts = get_option( 'pm_rvc_options' );
		$selected = '';
		?>
		<div id="pm_rvc_shortcode_modal_dialog" class="dialog" style="display: none;">
			<p>Select Tracker To Use</p>
			<?php pm_rvc_tracker_selector( $pm_rvc_opts, 'pm_rvc_trackr_id_select', 'pm_rvc_trackr_id_select', $selected ); ?>
			&nbsp;&nbsp;&nbsp;<a class="button button-primary" id="pm_rvc_trackr_insert">Insert</a>
		</div>
		<script>
		jQuery(document).ready( function($){
			$('#pm_rvc_trackr_insert').on('click', function(e, checked) {
				e.preventDefault();
				var selected_tracker = $('#pm_rvc_trackr_id_select :selected').val();
				if( selected_tracker == '0') {				
					var shortcode = '';
				} else {
					var shortcode = '[pm_revisitorc trackr_id="'+selected_tracker+'"]';
				}

				tinymce.activeEditor.execCommand('mceInsertContent', false, shortcode);
				top.tinymce.activeEditor.windowManager.close();
			});
		}); // global end
		</script>
		<?php
		}
	}
}
$shortcode = new Pm_Rvc_Shortcode;
$shortcode->init();