<?php
add_action( 'wp_ajax_pm_rvc_do_ajax_request', 'pm_rvc_process_ajax' );
function pm_rvc_process_ajax() {
	// Verify
	if( !isset( $_POST['nonce'] ) || !wp_verify_nonce( $_POST['nonce'], 'pm_rvc_add_trackr_content' ) )
		die( 'Nonce not present or valid' );
	if( ( $_POST['is_admin'] != 1 ) || ( $_POST['is_admin'] != true ) )
		die( 'You do not have admin privileges to perform this action' );
	$pm_rvc_opts = get_option( 'pm_rvc_options' );
	$count = ( isset( $_POST['count'] ) ) ? ++$_POST['count'] : '0' ;
	$custom_visitor = ( empty( $pm_rvc_opts['custom_visitor'] ) ) ? '' : $pm_rvc_opts['custom_visitor'] ;
	$content = '';
	$editor_settings = array(
		'textarea_name' => 'pm_rvc_options[trackr_content][]',
		'textarea_rows' => '5',
		'editor_class' => 'trackr_content',
		'teeny' => true,
		'drag_drop_upload' => true
		);
	?>
	<tr class="repeat" data-count="<?php echo $count; ?>">
		<td class="content-item">
			<select name="pm_rvc_options[trackr_id][]" class="trackr_id">
			<option value="0">Select Tracker</option>
			<?php if( !empty( $pm_rvc_opts['returning'] ) ) { ?>
				<option value="returning">Returning</option>
			<?php } if( !empty( $pm_rvc_opts['subscribers'] ) ) { ?>
				<option value="subscribers">Subscriber</option>
			<?php } if( !empty( $pm_rvc_opts['customers'] ) ) { ?>
				<option value="customers">Customers</option>
			<?php } ?>
			<?php
			if( !empty( $custom_visitor ) ) {
			foreach ($custom_visitor as $key => $item) { ?>
				<option value="<?php echo stripslashes( $item['id'] ); ?>"><?php echo stripslashes( $item['name'] ); ?></option>
			<?php } 
			}?>
			</select>
			<p>
				<a class="get_url button secondary-button" data-site-url="<?php echo site_url(); ?>" data-trackrid=""><?php _e( 'Get Tracking URL', 'pm-revisitor-content' ); ?></a>
			</p>
		</td>
		<td class="content-item">
		<?php wp_editor( $content, 'trackr_content_'.$count, $editor_settings ); ?>
		</td>
		<td class="content-item">
			<input type="text" class="small-text" name="pm_rvc_options[trackr_cookie][]" value="14"> 
			<label><?php _e( 'days', 'pm-revisitor-content' ); ?></label>
		</td>
		<td class="content-item">
			<a class="repeatable-trackr-remove button secondary-button">Remove</a>
		</td>
	</tr>
	<?php
	// Don't forget this or you'll get a 0 added to response
	die();
}

// Intercept before headers are sent
add_action( 'init', 'pm_rvc_init' );
function pm_rvc_init() {
	// Only run on front end
	if( !is_admin() ) {
		// Checks if URL contains pmrvc requests
		if( isset( $_GET['pmrvc'] ) ) {
			// If cookie & override present, unset cookie, set new cookie
			if( isset( $_COOKIE['pmrvc'] ) && isset( $_GET['override'] ) ) {
				// Get current cookie values
				$cookie_values = json_decode(stripslashes($_COOKIE['pmrvc']));
				$old_trackr_id = $cookie_values->trackr_id;
				$cookie_array = array( 
					'set' => 1,
					'trackr_id' => $old_trackr_id,
					'expire' => ''
				);
				// Unset old cookie
				setcookie( 'pmrvc', json_encode( $cookie_array ), time()-1, '/' );

				// Set and sanitize trackr_id
				$trackr_id = esc_attr( $_GET['pmrvc'] );
				pm_rvc_set_cookie( $trackr_id );
			} elseif( !isset( $_COOKIE['pmrvc'] ) ) {
				// If no cookie, set cookie
				// Set and sanitize trackr_id
				$trackr_id = esc_attr( $_GET['pmrvc'] );
				pm_rvc_set_cookie( $trackr_id );
			}
			// Otherwise, do nothing
		} else {
			$pm_rvc_opts = get_option( 'pm_rvc_options' );
			// Check if autocookie is turned on
			$autoreturn = ( empty( $pm_rvc_opts['autoreturn'] ) ) ? '' : esc_attr( $pm_rvc_opts['autoreturn'] );

			// Check autoreturn setting and if cookie already present
			if( $autoreturn === '1' && !isset( $_COOKIE['pmrvc'] ) ) {
				// Set cookie
				pm_rvc_set_cookie( 'returning' );
			}
			// Do nothing otherwise
		}
	}

	// Register shortcake
	$pm_rvc_opts = get_option( 'pm_rvc_options' );

	// Add default trackers if any
	if( !empty( $pm_rvc_opts['returning'] ) ) {
		$options['returning'] = 'Returning';
	} if( !empty( $pm_rvc_opts['subscribers'] ) ) {
		$options['subscribers'] = 'Subscriber';
	} if( !empty( $pm_rvc_opts['customers'] ) ) {
		$options['customers'] = 'Customers';
	}
	
	// List out custom trackers
	$tracker_array = $pm_rvc_opts['custom_visitor'];
	if( !empty( $tracker_array ) ) {
		foreach ($tracker_array as $tracker) {
			$trackr_id = esc_attr( $tracker['id'] );
			$trackr_name = esc_attr( $tracker['name'] );
			$options[$trackr_id] = $trackr_name;
		}
		}
	$options = ( empty($options) ) ? array() : $options;
	shortcode_ui_register_for_shortcode(
		'pm_revisitorc',
		array(
			// Display label
			'label' => PM_RVC_PLUGIN_NAME,
			// Icon for shortcode
			'listItemImage' => 'dashicons-editor-quote',
			// Shortcode attributes
			'attrs' => array(
				array(
					'label' => PM_RVC_PLUGIN_NAME,
					'attr' => 'trackr_id',
					'type' => 'select',
					'options' => $options
					)
				)
			)
		);
}

// Retrieve settings array based on trackr_id
function pm_rvc_get_tracker_data( $trackr_id ) {
	$pm_rvc_opts = get_option( 'pm_rvc_options' );
	foreach ( $pm_rvc_opts['tracker_content'] as $tracker_data ) {
		if( $tracker_data['id'] === $trackr_id ) {
			return $tracker_data;
			break;
		}
	}
}

// Function to set cookie
function pm_rvc_set_cookie( $trackr_id ) {
	// Get cookie duration
	$cookie_duration = pm_rvc_get_tracker_data( $trackr_id )['cookie'];
	// Build value JSON string
	$cookie_array = array( 
		'set' => 1,
		'trackr_id' => $trackr_id,
		'expire' => time()+3600*24*$cookie_duration
	);
	$value = json_encode( $cookie_array );
	// Set new cookie
	setcookie( 'pmrvc', $value, time()+3600*24*$cookie_duration, '/' );
}

// Function to output tracker selector drop down
function pm_rvc_tracker_selector( $pm_rvc_opts, $pm_rvc_select_name, $pm_rvc_select_id, $selected ) {
	?>
	<select name="<?php echo $pm_rvc_select_name; ?>" id="<?php echo $pm_rvc_select_id; ?>">
		<option value="0">Select Tracker</option>
		<?php if( !empty( $pm_rvc_opts['returning'] ) ) { ?>
			<option value="returning" <?php selected( $selected, 'returning', true ); ?>>Returning</option>
		<?php } if( !empty( $pm_rvc_opts['subscribers'] ) ) { ?>
			<option value="subscribers" <?php selected( $selected, 'subscribers', true ); ?>>Subscriber</option>
		<?php } if( !empty( $pm_rvc_opts['customers'] ) ) { ?>
			<option value="customers" <?php selected( $selected, 'customers', true ); ?>>Customers</option>
		<?php } ?>
	<?php
	if( !empty( $pm_rvc_opts['custom_visitor'] ) ) {
		foreach ($pm_rvc_opts['custom_visitor'] as $tracker) {
			?>
			<option value="<?php echo esc_attr( $tracker['id'] ); ?>" <?php selected( $selected, $tracker['id'], true ); ?>><?php echo esc_attr( $tracker['name'] ); ?></option>
			<?php
		}
		}
	?>
	</select>
	<?php
}