<?php
/* References
 * https://github.com/chipbennett/oenology/blob/master/functions/options.php#L84
 * http://www.chipbennett.net/2011/02/17/incorporating-the-settings-api-in-wordpress-themes/7/
 */
/* Add admin menu */
add_action( 'admin_menu', 'pm_rvc_admin_menu' );
function pm_rvc_admin_menu() {
	global $pm_rvc_settings;
	
	// Create options page
	$pm_rvc_settings = add_options_page(PM_RVC_PLUGIN_NAME.' Settings', PM_RVC_PLUGIN_NAME, 'manage_options', 'pm_rvc_settings', 'pm_rvc_options_page');
  	
  	// Load admin scripts when above page is called
  	add_action( 'load-'.$pm_rvc_settings, 'pm_rvc_adminscripts' );
}

// Enqueue admin scripts
function pm_rvc_adminscripts() {
	global $pm_rvc_settings;
	$screen = get_current_screen();

	// Load contextual help
	if( $screen->id != $pm_rvc_settings )
		return;

	// Enqueue scripts and styles
	wp_enqueue_script( 'pm-rvc-admin', PM_RVC_DIR.'js/admin.js', array('jquery'), PM_RVC_VERSION, false );
	wp_enqueue_script( 'pm-rvc-magnific', PM_RVC_DIR.'libs/magnific/magnific-popup.js', array('jquery', 'pm-rvc-admin'), PM_RVC_VERSION, false );
	wp_enqueue_style( 'pm-rvc-admin', PM_RVC_DIR.'css/admin.css', '', PM_RVC_VERSION, 'all' );

	// For Ajax
	$params = array(
		'nonce' => wp_create_nonce( 'pm_rvc_add_trackr_content' ),
		'is_admin' => is_admin(),
		'site_url' => site_url()
		);
	wp_localize_script( 'pm-rvc-admin', 'pm_rvc_object', $params );
}

function pm_rvc_print_footer_scripts() {
    $pointer_content = '<h3>'.PM_RVC_PLUGIN_NAME.'</h3>';
    $pointer_content .= __( 'Configure your settings here.', 'pm-revisitor-content' );
?>
   <script type="text/javascript">
   //<![CDATA[
   jQuery(document).ready( function($) {
    //jQuery selector to point to 
    $('#menu-settings').pointer({
        content: '<?php echo $pointer_content; ?>',
        position: 'top',
        close: function() {
            // This function is fired when you click the close button
            $.post (ajaxurl, {
		        pointer: 'pm_rvc_settings_dismiss',
		        action: 'dismiss-wp-pointer'
		    });
        }
      }).pointer('open');
   });
   //]]>
   </script>
<?php
}

// Output the settings page
function pm_rvc_options_page() {
	$tab = ( isset( $_GET['tab'] )  ? $_GET['tab'] : 'defaults' );
	$settings_section = 'pm_rvc_settings_'.$tab.'_tab';
	?>
	<div class="wrap">
		<h2><?php echo PM_RVC_PLUGIN_NAME.' '.__( 'Settings', 'pm-revisitor-content' ); ?></h2>
		<?php settings_errors( 'pm_rvc_options' ); ?>
		<?php pm_rvc_settings_page_tabs(); ?>

		<form action='options.php' method='post'>
			<?php 
			// Call functions to build form
			settings_fields( 'pm_rvc_options' );
			//do_settings_sections( 'pm_rvc_settings' );
			do_settings_sections( $settings_section );

			// Update settings based on tab			
			?>
			<p class="submit">
			<?php
			submit_button( __( 'Save Settings', 'pm-revisitor-content' ), 'primary', 'pm_rvc_options[submit-'.$tab.']', false, array( 'id' => 'submit-'.$tab ) );
			submit_button( __( 'Reset', 'pm-revisitor-content' ), 'secondary', 'pm_rvc_option[reset-'.$tab.']', false, array( 'id' => 'reset-'.$tab ) );
			?></p>
		</form>
	</div>
	<?php
}

// Create tabs
function pm_rvc_get_settings_page_tabs() {
	$tabs = array(
		'defaults' => __( 'Visitors', 'pm-revisitor-content' ),
		'tcontent' => __( 'Visitor Content', 'pm-revisitor-content' ),
		'misc' => __( 'Misc', 'pm-revisitor-content' )
		);
	return $tabs;
}

// HTML Markup for tabs
function pm_rvc_settings_page_tabs( $current = 'defaults' ) {
	// Determind which tab to display
	if( isset( $_GET['tab'] ) ) :
		$current = $_GET['tab'];
	else:
		$current = 'defaults';
	endif;

	// Get list of tabs
	$tabs = pm_rvc_get_settings_page_tabs();
	$links = array();

	// Create links for each tab
	foreach ($tabs as $tab => $name) :
		if( $tab == $current ) :
			$links[] = '<a class="nav-tab nav-tab-active" href="?page=pm_rvc_settings&tab='.$tab.'">'.$name.'</a>';
		else :
			$links[] = '<a class="nav-tab" href="?page=pm_rvc_settings&tab='.$tab.'">'.$name.'</a>';
		endif;
	endforeach;

	echo '<h2 class="nav-tab-wrapper">';
	foreach ( $links as $link ) {
		echo $link;
	}
	echo '</h2>';
}

// Register settings
add_action( 'admin_init', 'pm_rvc_initialize_settings' );
function pm_rvc_initialize_settings() {
	// Register fields
	register_setting( 
		'pm_rvc_options',
		'pm_rvc_options', 
		'pm_rvc_validate_options' );

	// Add settings sections
	pm_rvc_add_settings_sections();
	//add_settings_section( 'one', 'One Section', 'pm_rvc_one_section_callback', 'pm_rvc_settings' );

	// Add settings fields
	pm_rvc_add_settings_fields();
	//add_settings_field( 'textfield', 'Text Field', 'pm_rvc_textfield_field_callback', 'pm_rvc_settings', 'one' );
}

/*---------------
 Sections
 ---------------*/
// Register multiple settings (called in admin_init)
function pm_rvc_add_settings_sections() {
	// Array with settings identifiers
	$sects = array(
		'builtin_trackers' => array(
			'id' => 'builtin_trackers',
			'title' => __( 'Built-in Visitor', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_defaults_tab'
			),
		'autoreturn' => array(
			'id' => 'autoreturn',
			'title' => __( 'Auto Return Visitor', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_defaults_tab'
			),
		'custom_trackers' => array(
			'id' => 'custom_trackers',
			'title' => __( 'Custom Visitor', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_defaults_tab'
			),
		'tracker_content' => array(
			'id' => 'tracker_content',
			'title' => __( 'Visitor Content Snippet', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_tcontent_tab'
			),
		'data' => array(
			'id' => 'data',
			'title' => 'Data Handling',
			'page' => 'pm_rvc_settings_misc_tab'
			),
		);

	// Loop through each
	foreach ($sects as $s) {
		add_settings_section( 
			$s['id'].'_section',
			$s['title'].' Options',
			'pm_rvc_'.$s['id'].'_section_callback',
			$s['page']
			);
	}
}

/*----------------
Sections callback
-----------------*/
// Section callback
function pm_rvc_builtin_trackers_section_callback() {
	_e( 'At a general level, select who to track.', 'pm-revisitor-content' );
}
function pm_rvc_autoreturn_section_callback() {
	_e( 'Automatically sets anyone without a cookie as a return visitor. If this is not checked, you must send people through the return tracking URL before they will be recognized as a return visitor. Only works for return visitors', 'pm-revisitor-content' );
}
function pm_rvc_custom_trackers_section_callback() {
	_e( 'Set up additional, custom trackers if any.', 'pm-revisitor-content' );
}
function pm_rvc_tracker_content_section_callback() {
	_e( 'Set up content for each visitor type.', 'pm-revisitor-content' );
}
function pm_rvc_data_section_callback() {
}

/*------------------
Fields
------------------*/
// Register multiple fields (called in admin_init)
function pm_rvc_add_settings_fields () {
	// Array with field info
	$fields = array(
		'visitor_type' => array(
			'id' => 'visitor_type',
			'title' => __( 'Track Visitors', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_defaults_tab',
			'section' => 'builtin_trackers_section'
			),
		'auto_return_visitor' => array(
			'id' => 'auto_return_visitor',
			'title' => __( 'Auto cookie', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_defaults_tab',
			'section' => 'autoreturn_section'
			),
		'add_visitor_type' => array(
			'id' => 'add_visitor_type',
			'title' => __( 'Visitor Types', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_defaults_tab',
			'section' => 'custom_trackers_section'
			),
		'trackr_content' => array(
			'id' => 'trackr_content',
			'title' => __( 'Content', 'pm-revisitor-content' ),
			'page' => 'pm_rvc_settings_tcontent_tab',
			'section' => 'tracker_content_section'
			),
		'optsdata' => array(
			'id' => 'optsdata',
			'title' => 'Settings Data',
			'page' => 'pm_rvc_settings_misc_tab',
			'section' => 'data_section'
			),
		);

	// Loop through each in array
	foreach ($fields as $f) {
		add_settings_field( 
			$f['id'],
			$f['title'],
			'pm_rvc_'.$f['id'].'_field_callback',
			$f['page'],
			$f['section']
			);
	}
}

/*--------------------
Fields callback
--------------------*/
function pm_rvc_optsdata_field_callback() {
	// Be sure the field names are formatted for array!
	// Also be sure name does not have ''
	$pm_rvc_opts = get_option( 'pm_rvc_options' );;
	$del_opt_settings = ( empty( $pm_rvc_opts['del_opt_settings'] ) ) ? '' : $pm_rvc_opts['del_opt_settings'];
	?>
	<input type="checkbox" name="pm_rvc_options[del_opt_settings]" value="1" id="del_opt_settings" <?php checked( $del_opt_settings, '1', true ); ?>> <label for="del_opt_settings"><?php _e( 'Yes. Delete all settings data when I uninstall', 'pm-revisitor-content' ); ?></label><br>
	<small><?php _e( 'Use with caution! When checked, all settings will be deleted on plugin uninstall. Disabling will not delete data only uninstall.', 'pm-revisitor-content' ); ?></small>
	<?php
}
function pm_rvc_visitor_type_field_callback() {
	$pm_rvc_opts = get_option( 'pm_rvc_options' );
	$returning = ( empty( $pm_rvc_opts['returning'] ) ) ? '' : $pm_rvc_opts['returning'];
	$subscribers = ( empty( $pm_rvc_opts['subscribers'] ) ) ? '' : $pm_rvc_opts['subscribers'];
	$customers = ( empty( $pm_rvc_opts['customers'] ) ) ? '' : $pm_rvc_opts['customers'];
	?>
	<input type="checkbox" name="pm_rvc_options[returning]" value="1" id="returning" <?php checked( $returning, 1, true ); ?>>
	<label for="returning"><?php _e( 'Return visitors', 'pm-revisitor-content' ); ?></label>  
	<input type="checkbox" name="pm_rvc_options[subscribers]" value="1" id="subscribers" <?php checked( $subscribers, 1, true ); ?>> 
	<label for="subscribers"><?php _e( 'Subscribers', 'pm-revisitor-content' ); ?></label>  
	<input type="checkbox" name="pm_rvc_options[customers]" value="1" id="customers" <?php checked( $customers, 1, true ); ?>> 
	<label for="customers"><?php _e( 'Customers', 'pm-revisitor-content' ); ?></label>
	<?php
}
function pm_rvc_auto_return_visitor_field_callback() {
	$pm_rvc_opts = get_option( 'pm_rvc_options' );
	$autoreturn = ( empty( $pm_rvc_opts['autoreturn'] ) ) ? '' : $pm_rvc_opts['autoreturn'];
	?>
	<input type="checkbox" name="pm_rvc_options[autoreturn]" value="1" id="autoreturn" <?php checked( $autoreturn, 1, true ); ?>>
	<label for="autoreturn"><?php _e( 'Yes', 'pm-revisitor-content' ); ?></label>
	<p id="autoreturn_message"><?php _e( 'This only works with return visitors. Please check Return visitors above.', 'pm-revisitor-content' ); ?></p>
	<?php
}
function pm_rvc_add_visitor_type_field_callback() {
	$pm_rvc_opts = get_option( 'pm_rvc_options' );
	$custom_visitor = ( empty( $pm_rvc_opts['custom_visitor'] ) ) ? '' : $pm_rvc_opts['custom_visitor'] ;
	?>
	<div class="custom-tracker-repeatable-wrap">
	<small><?php _e( 'Name field is for reference only. Do not enter spaces for the ID, underscores and dashes OK.', 'pm-revisitor-content'); ?></small>
		<ul class="repeatable-fields-list">
		<?php if( empty( $pm_rvc_opts['custom_visitor'] ) ) { ?>
			<li class="repeat">
				<label><?php _e( 'Name', 'pm-revisitor-content' ); ?></label>
				<input type="text" class="visitor-name" name="pm_rvc_options[name][]" value="">
				<label><?php _e( 'ID', 'pm-revisitor-content' ); ?></label>
				<input type="text" class="visitor-id" name="pm_rvc_options[id][]" value="">
				<a class="repeatable-remove button button-secondary"><?php _e('Remove', 'pm-revisitor-content'); ?></a>
			</li>
		<?php } else {
			foreach ($pm_rvc_opts['custom_visitor'] as $key => $item) {
				?>
				<li class="repeat">
					<label><?php _e( 'Name', 'pm-revisitor-content' ); ?></label>
					<input type="text" class="visitor-name" name="pm_rvc_options[name][]" value="<?php echo stripslashes( $item['name'] ); ?>">
					<label><?php _e( 'ID', 'pm-revisitor-content' ); ?></label>
					<input type="text" class="visitor-id" name="pm_rvc_options[id][]" value="<?php echo stripcslashes( $item['id'] ); ?>">
					<a class="repeatable-remove button button-secondary"><?php _e( 'Remove', 'pm-revisitor-content' ); ?></a>
				</li>
				<?php
			}
			} ?>
		</ul>
	<p><button id="add-custom-tracker" class="button button-secondary"><?php _e( 'Add Custom Visitor', 'pm-revisitor-content' ); ?></button></p>
	</div>
	<?php
}
function pm_rvc_trackr_content_field_callback() {
	$pm_rvc_opts = get_option( 'pm_rvc_options' );
	$custom_visitor = ( empty( $pm_rvc_opts['custom_visitor'] ) ) ? '' : $pm_rvc_opts['custom_visitor'] ;
	$editor_settings = array(
		'textarea_name' => 'pm_rvc_options[trackr_content][]',
		'textarea_rows' => '5',
		'editor_class' => 'trackr_content',
		'teeny' => true,
		'drag_drop_upload' => true
		);
	?>
	<table class="repeatable-fields-list" id="tracker-content-list">
	<tr>
		<th><?php _e( 'Visitor Type / URL', 'pm-revisitor-content' ); ?></th>
		<th><?php _e( 'Content', 'pm-revisitor-content' ); ?></th>
		<th><?php _e( 'Cookie', 'pm-revisitor-content' ); ?></th>
		<th><?php _e( 'Action', 'pm-revisitor-content' ); ?></th>
	</tr>
		<?php if( empty( $pm_rvc_opts['tracker_content'] ) ) { ?>
			<tr class="repeat" data-count="0">
				<td class="content-item">
					<select name="pm_rvc_options[trackr_id][]" class="trackr_id">
					<option value="0">Select Visitor</option>
					<?php if( !empty( $pm_rvc_opts['returning'] ) ) { ?>
						<option value="returning">Returning</option>
					<?php } if( !empty( $pm_rvc_opts['subscribers'] ) ) { ?>
						<option value="subscribers">Subscriber</option>
					<?php } if( !empty( $pm_rvc_opts['customers'] ) ) { ?>
						<option value="customers">Customers</option>
					<?php } ?>
					<?php
					if( !empty( $custom_visitor ) ) {
					foreach ($custom_visitor as $key => $item) { ?>
						<option value="<?php echo stripslashes( $item['id'] ); ?>"><?php echo stripslashes( $item['name'] ); ?></option>
					<?php } 
					}?>
					</select>
					<p>
						<a class="get_url button secondary-button" data-site-url="<?php echo site_url(); ?>" data-trackrid=""><?php _e( 'Get Tracking URL', 'pm-revisitor-content' ); ?></a>
					</p>
				</td>
				<td class="content-item">
				<?php wp_editor( '', 'trackr_content_0', $editor_settings ); ?>
				</td>
				<td class="content-item">
					<input type="text" class="small-text" name="pm_rvc_options[trackr_cookie][]" value="14"> 
					<label><?php _e( 'days', 'pm-revisitor-content' ); ?></label>
				</td>
				<td class="content-item">
					<a class="repeatable-trackr-remove button secondary-button"><?php _e( 'Remove', 'pm-revisitor-content' ); ?></a>
				</td>
			</tr>
		<?php } else {
			foreach ( $pm_rvc_opts['tracker_content'] as $k => $v ) {
				$content = stripslashes( $v['content'] );
				$trackr_cookie = ( empty( $v['cookie'] ) ) ? '14' : esc_attr( $v['cookie'] ) ;
			?>
			<tr class="repeat" data-count="<?php echo $k; ?>">
				<td class="content-item">
					<select name="pm_rvc_options[trackr_id][]" class="trackr_id">
					<option value="0">Select Visitor</option>
					<?php if( !empty( $pm_rvc_opts['returning'] ) ) { ?>
						<option value="returning" <?php selected( $v['id'], 'returning', true ); ?>>Returning</option>
					<?php } if( !empty( $pm_rvc_opts['subscribers'] ) ) { ?>
						<option value="subscribers" <?php selected( $v['id'], 'subscribers', true ); ?>>Subscriber</option>
					<?php } if( !empty( $pm_rvc_opts['customers'] ) ) { ?>
						<option value="customers" <?php selected( $v['id'], 'customers', true ); ?>>Customers</option>
					<?php } ?>
					<?php
					if( !empty( $custom_visitor ) ) {
					foreach ($custom_visitor as $key => $item) { ?>
						<option value="<?php echo stripslashes( $item['id'] ); ?>" <?php selected( $v['id'], $item['id'], true ); ?>><?php echo stripslashes( $item['name'] ); ?></option>
					<?php } 
					}?>
					</select>
					<p>
						<a class="get_url button secondary-button" data-site-url="<?php echo site_url(); ?>" data-trackrid="<?php echo esc_attr( $v['id'] ); ?>"><?php _e( 'Get Tracking URL', 'pm-revisitor-content' ); ?></a>
					</p>
				</td>
				<td class="content-item">
				<?php wp_editor( $content, 'trackr_content_'.$k, $editor_settings ); ?>
				</td>
				<td class="content-item">
					<input type="text" class="small-text" name="pm_rvc_options[trackr_cookie][]" value="<?php echo $trackr_cookie; ?>"> 
					<label><?php _e( 'days', 'pm-revisitor-content' ); ?></label>
				</td>
				<td class="content-item">
					<a class="repeatable-trackr-remove button secondary-button"><?php _e('Remove', 'pm-revisitor-content'); ?></a>
				</td>
			</tr>
		<?php }
		} ?>
	</table>
	<p><button id="add-tracker-content" class="button button-secondary"><?php _e( 'Add Content', 'pm-revisitor-content' ); ?></button></p>
	<div id="popup_trackr_url" class="white-popup mfp-hide">
		<p><?php _e( 'Copy this URL', 'pm-revisitor-content' ); ?><br>
			<input type="text" id="copy_trackr_url" class="large-text">
		</p>
		<p><label><input type="checkbox" id="trackr_url_override"> <?php _e( 'Override', 'pm-revisitor-content' ); ?></label></p>
	</div>
	<?php
}

// Validate options
function pm_rvc_validate_options( $input ) {
	$opts = get_option( 'pm_rvc_options' );
	$valid_input = $opts;

	// Determine which tab is submitted
	$defaults = ( !empty( $input['submit-defaults'] ) ? true : false );
	$tcontent = ( !empty( $input['submit-tcontent'] ) ? true : false );
	$misc = ( !empty( $input['submit-misc'] ) ? true : false );

	if( $defaults ) {
		// validate only those settings
		$valid_returning = ( isset( $input['returning'] ) && $input['returning'] == '1' ) ? '1' : '0' ;
		$valid_subscribers = ( isset( $input['subscribers'] ) && $input['subscribers'] == '1' ) ? '1' : '0' ;
		$valid_customers = ( isset( $input['customers'] ) && $input['customers'] == '1' ) ? '1' : '0' ;
		$valid_autoreturn = ( isset( $input['autoreturn'] ) && $input['autoreturn'] == '1' ) ? '1' : '0' ;
		$name = ( isset( $input['name'] ) ) ? $input['name'] : '' ;
		$id = ( isset( $input['id'] ) ) ? $input['id'] : '' ;
		$count = count( $name );

		if( !empty( $name ) ) {
			for( $i = 0; $i < $count; $i++ ) {
				if( !empty( $name[$i] ) ) {
					$valid_custom_visitor[$i]['name'] = sanitize_text_field( $name[$i] );
					$valid_custom_visitor[$i]['id'] = sanitize_text_field( $id[$i] );
				}
			}
		} else {
			$valid_custom_visitor = array();
		}

		// Build array to save
		$valid_input = array(
			'returning' => $valid_returning,
			'subscribers' => $valid_subscribers,
			'customers' => $valid_customers,
			'autoreturn' => $valid_autoreturn,
			'custom_visitor' => $valid_custom_visitor,
			'tracker_content' => $opts['tracker_content'],
			'del_opt_settings' => $opts['del_opt_settings']
		);
	} elseif ( $tcontent ) {
		// validate only those settings
		$trackr_id = ( isset( $input['trackr_id'] ) ) ? $input['trackr_id'] : '' ;
		$trackr_content = ( isset( $input['trackr_content'] ) ) ? $input['trackr_content'] : '' ;
		$trackr_cookie = ( isset( $input['trackr_cookie'] ) ) ? $input['trackr_cookie'] : '' ;
		$trackr_count = count( $trackr_id );

		if( !empty( $trackr_id ) ) {
			for( $i = 0; $i < $trackr_count; $i++ ) {
				if( !empty( $trackr_id[$i] ) ) {
					$valid_tracker_content[$i]['id'] = sanitize_text_field( $trackr_id[$i] );
					$valid_tracker_content[$i]['content'] = wp_kses_post( $trackr_content[$i] );
					$valid_tracker_content[$i]['cookie'] = intval( $trackr_cookie[$i] );
				}
			}
		} else {
			$valid_tracker_content = array();
		}

		// Build array to save
		$valid_input = array(
			'returning' => $opts['returning'],
			'subscribers' => $opts['subscribers'],
			'customers' => $opts['customers'],
			'autoreturn' => $opts['autoreturn'],
			'custom_visitor' => $opts['custom_visitor'],
			'tracker_content' => $valid_tracker_content,
			'del_opt_settings' => $opts['del_opt_settings']
			);
	} elseif ( $misc ) {
		// validate only those settings
		$valid_del_opt_settings = ( isset( $input['del_opt_settings'] ) && $input['del_opt_settings'] == '1' ) ? '1' : '0' ;

		// Use saved settings for options not being validated
		//$add = ( empty( $pm_lgate_opts['add'] ) ) ? '' : $pm_lgate_opts['add'] ;

		// Build array to save
		$valid_input = array(
		'returning' => $opts['returning'],
		'subscribers' => $opts['subscribers'],
		'customers' => $opts['customers'],
		'autoreturn' => $opts['autoreturn'],
		'custom_visitor' => $opts['custom_visitor'],
		'tracker_content' => $opts['tracker_content'],
		'del_opt_settings' => $valid_del_opt_settings
		);
	}
	
	return $valid_input;
	/*print_r($input);
	echo "<hr>";
	print_r($valid_input);
	exit(); // For testing*/
}